clear
close all

% EKM_nonlinear_supercomp_selection

cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments

%% datasets
dataset = 6;    % 1 - colon cancer Alon PNAS 1999; 
               % 2 - leukemia Armstrong Nature Genetics 2002;
               % 3 - lung cancer - PNAS 2001
               % 4 - 14 tummors,12 normal - PNAS 2001
               % 5 - diffusive B cell lymphoma (DBCL), Nature Medicine 2002
               % 6 - prostate cancer
               % 7 - leukemia Golub Science 1999
               % 8 - MS ovarian cancer Petricoin et al. The Lancet 2002 
               % 9 - MS prostate cancer Petricoin et al. J. Nat Canc Inst 2002

%% mapping parameters
kernel = 'exponential';     % Gaussian kernel EKM of order D with variance sigma^2;
                         % polynomial kernel EKM of order D;
                     
D = 20;               % order of EKM
b = 0;               % for appoximate polynomial EFM
var1 = 1.0e0;        % variance of Gaussian kernel or degree of polynomial kernel
var2 = 1;            % offset of polynomial kernel
                
sca_alg = 6;         % 1-linprog; 2-Boyd's interior-point method; 3-l2 reg l1; 4-OMP greedy; 5-IRLS; 6-BATCH shrinkage; 7 - Smooth L0
 
if sca_alg == 6
    lambda = 0.2;
end

if(sca_alg == 5)
   p_norm = 0.1;
end

%% processing
if dataset == 1
    cd .\Colon_cancer
    load gene_profiles_colon_ordered.txt
    [T N] = size(gene_profiles_colon_ordered);
    X = gene_profiles_colon_ordered;
    xm = max(max(abs(X)));
    X = X/xm;
    [p N]=size(X);
    control_ref = zeros(1,p);
    for i=1:22
        control_ref = control_ref + (1/22)*X(:,i)';   % Normal
    end
    cancer_ref = zeros(1,p);
    for i=1:40
        cancer_ref = cancer_ref + (1/40)*X(:,i+22)';  % Cancer
    end
elseif dataset == 5
    cd .\Diffuse_B_cell_lymphoma
    load DLBCL
    [N T] = size(data);
    X = data(:,2:T)';
    xm = max(max(abs(X)));
    X = X/xm;
    [p N]=size(X);
    control_ref = zeros(1,p);
    for i=1:58
        control_ref = control_ref + (1/58)*X(:,i)';     % DLBCL
    end
    cancer_ref = zeros(1,p);
    for i=1:19
        cancer_ref = cancer_ref + (1/19)*X(:,i+58)';    % FL
    end
elseif dataset == 6
    cd .\Prostate_cancer_gene_profiles
    load Prostate_Tumor
    [N T] = size(data);
    X = data(:,2:T)';
    xm = max(max(abs(X)));
    X = X/xm;
    [p N]=size(X);
    control_ref = zeros(1,p);
    for i=1:50
        control_ref = control_ref + (1/50)*X(:,i)';     % Normal
    end
    cancer_ref = zeros(1,p);
    for i=1:52
        cancer_ref = cancer_ref + (1/52)*X(:,i+50)';    % Cancer
    end
elseif dataset == 8
    cd D:\MATLAB\work\GSK\Ovarian_Data_WCX2_CSV\control\Group_C    % Low resolution SELDI TOF MS data
    load Control_C
    [T N] = size(Control_C);
    X = Control_C(3:T,2:51);
    cd D:\MATLAB\work\GSK\Ovarian_Data_WCX2_CSV\control\Group_D    % Low resolution SELDI TOF MS data
    load Control_D
    X(:,51:100) = Control_D(3:T,2:51);

    cd D:\MATLAB\work\GSK\Ovarian_Data_WCX2_CSV\cancer\Group_A    % Low resolution SELDI TOF MS data
    load Cancer_A
    X(:,101:150) = Cancer_A(3:T,2:51);

    cd D:\MATLAB\work\GSK\Ovarian_Data_WCX2_CSV\cancer\Group_B    % Low resolution SELDI TOF MS data
    load Cancer_B
    X(:,151:200) = Cancer_B(3:T,2:51);
    xm = max(max(abs(X)));
    X = X/xm;
    [p N]=size(X);
    control_ref = zeros(1,p);
    for i=1:100
        control_ref = control_ref + (1/100)*X(:,i)';     % Normal
    end
    cancer_ref = zeros(1,p);
    for i=1:100
        cancer_ref = cancer_ref + (1/100)*X(:,i+100)';    % Cancer
    end   
elseif dataset == 9
    cd D:\MATLAB\work\GSK\JNCI_Data_7_3_02\No_Evidence_of_Disease_PSA_Less_1    % No evidence of cancer, PSA less than 1
    load Control_PSA_less_1
    [T N] = size(Control_PSA_less_1);
    X = Control_PSA_less_1(:,1:63);

    cd D:\MATLAB\work\GSK\JNCI_Data_7_3_02\Prostate_Cancer_PSA_Greater_10    % Cancer, PSA greater than 10
    load Cancer_PSA_greater_10
    X(:,64:106) = Cancer_PSA_greater_10(:,1:43);

    cd D:\MATLAB\work\GSK\JNCI_Data_7_3_02\Prostate_Cancer_PSA_4_10
    load Cancer_PSA_4_10
    X(:,107:132)=Cancer_PSA_4_10(:,1:26);
    xm = max(max(abs(X)));
    X = X/xm;
   [p N]=size(X);
    control_ref = zeros(1,p);
    for i=1:63
        control_ref = control_ref + (1/63)*X(:,i)';     % Normal
    end
    cancer_ref = zeros(1,p);
    for i=1:69
        cancer_ref = cancer_ref + (1/69)*X(:,i+63)';    % Cancer
    end   
end
 
Xtz(1,:)=cancer_ref;
Xtc(1,:)=cancer_ref;

if dataset == 1
    cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Colon_cancer\Exp_EKM_D_20
elseif dataset == 5
    cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Diffuse_B_cell_lymphoma\Gauss_EKM_D_5
elseif dataset == 6
    cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Prostate_cancer_gene_profiles\Exp_EKM_D_20
   % cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Prostate_cancer_gene_profiles\linear
elseif dataset == 8
    cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Ovarian_cancer_MS\Exp_EKM_D_20
elseif dataset == 9
    cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Prostate_cancer_MS\Exp_EKM_D_20
end

for nz=1:N
    nz
   % *********************************************************************
    %  Reference cancer
    %**********************************************************************
 
    Xtz(2,:)= X(:,nz)';
    
    % basis learning
    % eliminate small components
        Xn=sqrt(sum(Xtz.*Xtz));
        [xmax indmax] = max(Xn);
        Xn = Xn./xmax;
        P=Xn-1/100000; P=sign(P); P=(-P+1)/2;
        Xnp=Xn.*P;
        [xd,Ind]=setdiff(Xn,Xnp);
        %XR = Xtz(:,Ind);
        XR = Xtz;
    
        cnum=1e3;
        if (D <= 20)
             epscnum=150.0;
        else
             epscnum=50.0;
        end
    
        while cnum > epscnum
           [SH,B]=kmeans(XR',D,'distance','sqEuclidean','Replicates',1,'EmptyAction','singleton'); %,'start','sample');
           V = B';
           cnum=cond(V)
        end
        clear SH
        
        V(:,D+1)=[1 0]';
        
        FIX = empirical_kernel_map(Xtz,V,kernel,0,var1,var2); 

        [Dd TL]=size(FIX);
   
    % estimation of basis matrix by clustering (Fast separable NMF
    % algorithm)
    
    [J,normM,U] = FastSepNMF(FIX,D+1,0);   % for determined problems
    AH=FIX(:,J);
    M = size(AH,2);

    % reference sample position in RKHS
    er=zeros(D,1);
    er(D+1,1)=1;

    for m=1:M
        tmp(m)= AH(:,m)'*er/norm(AH(:,m));  % cosine of the angle
    end
    [angm ind_c] = max(tmp);
    clear tmp

    f = ones(M,1);
    lb = zeros(M,1);
    ub = 1e6*ones(M,1);

    if sca_alg == 6
       tic
      % lambda = 0.2;
       lam = lambda * ones(1, TL);
       List = 2 * eigs(AH' * AH, 1) * ones(1, TL);
       maxiter = 100;
       SH = iterative_shrinkage_l2_batch(AH, FIX, M,'lambda', lam/2, 'maxiter', maxiter,'L', List);
       toc
    elseif sca_alg == 7
        sigma_off = 0.0005;
        % Important Algorithm's Parameters
        sigma_decrease_factor = 0.9;
        L = 3;
        if sigma_off>0
           sigma_min = sigma_off*4;
        else
           sigma_min = 0.00001;
        end
        % Global Init:
        mu_0 = 0.1;
        SH=SL0_MMV(AH,FIX,sigma_min,sigma_decrease_factor,mu_0,L,pinv(AH));
    else
        tic
        g = waitbar(0,'Please wait...');
        for m=1:TL
            cxm=FIX(:,m);
            if(sca_alg == 1)
                [sh,fval,exitflag,output]=linprog(f,[],[],AH,cxm,lb,ub,[],optimset('Display','iter'));    % solves linear program min(x) f'*x s.t. Ax=b, lb<=x<=ub.
            elseif (sca_alg == 2)
                lam = 1e-2*find_lambdamax_l1_ls(AH',cxm);
                tol=1e-6;        % for interior point method
                [sh,status]=l1_ls(AH,cxm,lam,tol);
            elseif (sca_alg == 3)
                [sh]=GPSR_BB(cxm,AH,0.1);  % M.A.T. Figueiredo, R.D.Nowak, S.J. Wright, IEEE J Sel. Toop. Sig. Proc, 2007.
            elseif (sca_alg == 4)
                [sh,err,ittime]=greed_omp(cxm,AH,4); % Sparsify package by T. Blumensath
            elseif (sca_alg == 5)           
                sh=irls(cxm,AH,p_norm);
            end
            SH(:,m) = sh;
            waitbar(m/TL,g)
        end
        close(g);
        toc
    end
    Supercomp_cancer_ref_control(nz,:)=SH(ind_c,:);
    clear SH
    
   % save Supercomp_cancer_ref_control Supercomp_cancer_ref_control
end

figure(1)
title('Cancer ref. cancer')
for n=1:62
    subplot(8,8,n)
    plot(Supercomp_cancer_ref_control(n,:))
    set(gca,'Ytick',[]);
    set(gca,'Xtick',[]);
end

for i=1:p
    var_gen(i)=var(Supercomp_cancer_ref_control(:,i));
end

figure
stem(var_gen)

[var_sort ind_sort] = sort(var_gen,'descend');

name=strcat('sorted_variance_cancer_cancer_lam_0p',num2str(100*lambda));
save(name,'var_sort')
name=strcat('sorted_indices_cancer_cancer_lam_0p',num2str(100*lambda));
save(name,'ind_sort')

%% for mass spectra only !!!!!!!!!!

if dataset > 7
    % initiaize peak detector
    chain_order = 3;
    if (rem(chain_order,2) == 0)
        chain_order = chain_order + 1;
    end
    chain = zeros(1,chain_order)';
    mid=round(chain_order/2);
    chain(mid)=-0.1;

    M=0;
    for i=1:length(var_gen)
        chain(1)=var_gen(i);
        [fm indm]=max(chain);
        if indm == mid
            M=M+1;
            ind(M)=i-mid+1;
            var_peak(M)=var_gen(i-mid+1);
        end
        chain=circshift(chain,1);
    end

    [var_peak_sort ind_sort] = sort(var_peak,'descend');
    ind_peak_sort=ind(ind_sort);

    name=strcat('sorted_peaked_variance_cancer_cancer_lam_0p',num2str(100*lambda));
    save(name,'var_peak_sort')
    name=strcat('sorted_peaked_indices_cancer_cancer_lam_0p',num2str(100*lambda));
    save(name,'ind_peak_sort')
end

